import logging
import os.path
from logging.handlers import RotatingFileHandler
from typing import Callable


class CustomLogger:
    def __init__(
        self,
        name: str,
        logging_level: str = "DEBUG",
        logs_dir: str = "logs",
        max_MB: int = 5,
        max_count: int = 10,
        format_string: str = "%(asctime)s [%(levelname)s] (%(module)s.%(funcName)s:%(lineno)d) - %(message)s",
        args_separator: str = " | "
    ):
        super().__init__(name)

        logging_level = logging.getLevelName(logging_level.upper())
        os.makedirs(logs_dir)
        logs_path = os.path.join(logs_dir, f"{name}.log")
        self.args_separator = args_separator

        handler = RotatingFileHandler(logs_path, maxBytes=max_MB * 1024 * 1024, backupCount=max_count)
        # handler = logging.FileHandler(f"{NAME}.log", mode="w")
        formatter = logging.Formatter(format_string)

        handler.setFormatter(formatter)
        self._logger = logging.getLogger(name)
        self._logger.setLevel(logging_level)
        self._logger.addHandler(handler)

    def debug(self, message: str, *args) -> None:
        self._log(self._logger.debug, message, *args)

    def info(self, message: str, *args) -> None:
        self._log(self._logger.info, message, *args)

    def warning(self, message: str, *args) -> None:
        self._log(self._logger.warning, message, *args)

    def error(self, message: str, *args) -> None:
        self._log(self._logger.error, message, *args)

    def critical(self, message: str, *args) -> None:
        self._log(self._logger.critical, message, *args)

    def _log(self, level_func: Callable, message: str, *args) -> None:
        if args:
            message += self.args_separator.join(args)
        level_func(message)
